#include "./config.h"//包含用户配置库

uint Temp_Val;//温度值

#define DQ P43//定义单总线引脚

//单总线变量初始化函数
void DS18B20_Tag_Init(void) {
    Temp_Val=0;//温度值初始化
}

//单总线延时函数
void delay_us(uint32_t us) {
    unsigned long edata i;

    while(--us) {
        _nop_();
        _nop_();
        _nop_();
        i = 4UL;
        while (i) i--;        
    }
}

//延时1ms函数
void DS18B20_Delay_ms(uint ms) {
	unsigned long edata i;
    while (ms--)
    {
        _nop_();
        _nop_();
        _nop_();
        i = 5998UL;
        while (i) i--;        
    }
}

//单总线初始化函数
bit DS18B20_Init(void) {
    static bit ack=0;//应答信号
    DQ = 0;//拉低总线
    delay_us(500);//延时480us以上
    DQ = 1;//释放总线
    delay_us(60);//延时大于60us,确保能收到DS18B20发送的60-240us的存在脉冲
    ack = DQ;//读取单总线上的状态
    delay_us(240);//延时240us,让DS18B20释放总线
    DQ=1;//释放总线
    return ack;//返回应答信号
}

//单总线写一个字节函数
void DS18B20_WriteByte(unsigned char dat) {
    u8 i;//计数变量

    for(i=0;i<8;i++) {
        DQ = 0;//拉低总线,产生写时隙
        delay_us(2);//每次读写都要间隔大于1us
        DQ=1;//15us内释放总线(写1时序)
        delay_us(2);//稍作延时
        DQ=dat&0x01;//往单总线上写数据(LSB)
        delay_us(45);//在15-60us内等待DS18B20采集数据
        DQ=1;//释放总线
        dat>>=1;//右移一位,准备下一次发送
    }
}

//单总线读一个字节函数
unsigned char DS18B20_ReadByte(void) {
    u8 i;//计数变量
    u8 dat;//读取的数据

    for(i=0;i<8;i++) {
        DQ = 0;//拉低总线,产生读时隙
        delay_us(2);//稍作延时
        DQ = 1;//释放总线
        dat>>=1;//右移一位,准备接收数据
        if(DQ) dat|=0x80;//读取数据(LSB),要读取最高位
        delay_us(60);//延时60us
        DQ=1;//释放总线
        delay_us(2);//稍作延时
    }
    return dat;
}

//温度测量函数
float Temp_Capture(void) {
    float Temp;//温度值
    uchar Temp_L;//温度值低8位
    uchar Temp_H;//温度值高8位

    DS18B20_Init();//初始化单总线
    DS18B20_WriteByte(0xCC);//跳过ROM
    DS18B20_WriteByte(0x44);//启动温度转换
    while(!DQ);//等待转换完成

    DS18B20_Init();//初始化单总线
    DS18B20_WriteByte(0xCC);//跳过ROM
    DS18B20_WriteByte(0xBE);//读取温度值
    Temp_H = DS18B20_ReadByte();//读取高8位
    Temp_L = DS18B20_ReadByte();//读取低8位
    Temp=(float)((Temp_H+Temp_L*256)*0.0625);//计算温度值
    return Temp;//返回温度值
}

////LCD温度显示函数
//void Temp_Show(void) {
//    static uchar xdata Temp_Text[100];//温度文字格式化数组
//    sprintf(Temp_Text,"Temperature:%u",Temp_Val);//格式化温度文字
//    LCD1602_Show_Str(0,2,Temp_Text);//显示温度文字
//}